//
//  ARRenderer.h
//  ObviousDemo
//
//  Copyright 2012 Obvious Engine. All rights reserved.
//

#ifndef AR_RENDERER_H
#define AR_RENDERER_H

#include <OpenGLES/ES1/gl.h>
#include <OpenGLES/ES1/glext.h>
#include <OpenGLES/ES2/gl.h>
#include <OpenGLES/ES2/glext.h>

// i'm using GLM for doing GL maths stuff. it seems to be a nice library, and 
// is necessary since GLES2.0 doesn't have any of the matrix stack of 1.0.
// http://glm.g-truc.net/
#include <glm/glm.hpp>

#import <ObviousEngine.h>

class ARRenderer
{
public:
    ARRenderer(int frameWidth, int frameHeight, const float* projectionMatrix, float stretchFactor = 1.f);
    ~ARRenderer();
    
    void setFrameImage(const unsigned char* y, const unsigned char* uv);
    void setModelviewMatrix(const float* modelviewMatrix);
    inline void setModelInfo(const OEModelInfo& info) { m_modelInfo = info; }
    inline void setHasDetection(bool hasDetection) { m_hasDetection = hasDetection; }
    inline void toggleDrawMode() { m_drawMode = 1 - m_drawMode; }
    
    void render();
    
    inline void flipTextureBuffers() { m_activeBuffer = 1-m_activeBuffer; }
    inline float getScale() const { return m_scale; }
    inline void setScale(float s) { m_scale = s; }
    
private:

    int m_frameWidth;
    int m_frameHeight;
    int m_halfFrameWidth;
    int m_halfFrameHeight;
    float m_stretchFactor;
    
    GLuint m_frameTextures[4];
    int m_activeBuffer;
    
    GLuint m_objectProgram;
    GLuint m_cameraProgram;
    GLuint m_warpProgram;
    
    GLint m_uniformObjectModelview;
    GLint m_uniformObjectProjection;
    GLint m_uniformObjectColour;
    GLint m_uniformCameraProjection;
    GLint m_uniformCameraTextureY;
    GLint m_uniformCameraTextureUV;
    GLint m_uniformWarpProjection;
    GLint m_uniformWarpModelview;
    GLint m_uniformWarpStretchFactor;
    GLint m_uniformWarpTextureY;
    GLint m_uniformWarpTextureUV;
    GLint m_uniformWarpTime;
    GLint m_uniformWarpScale;
    GLint m_uniformWarpIsCylindrical;
    
    enum Attributes
    {
        ATTRIBUTE_VERTEX,
        ATTRIBUTE_DISPLACEMENT,
    };
    
    OEModelInfo m_modelInfo;
    bool m_hasDetection;
    glm::mat4 m_cameraProjection;
    glm::mat4 m_objectProjection;
    glm::mat4 m_modelview;
    float m_scale;
    int m_t;
    int m_drawMode;
        
    void renderCamera();
    void renderCube();
    void renderDisplacementGrid();
    void renderCylinder();
    void renderCylinderWarp();
    void renderCylinderParticles();

    
    
    bool loadShaders();
    bool loadShaderPair(GLuint program, const char* vertShaderPath, const char* fragShaderPath);
    bool compileShader(GLuint* shader, GLenum type, const char* path);
    bool linkProgram(GLuint prog);
    bool validateProgam(GLuint prog);
    
};

#endif // AR_RENDERER_H